!*****************************************************************************

!  MPFUN20-Fort: A thread-safe arbitrary precision computation package
!  Global data definition module (module MPFUNA)

!  Revision date:  9 Jan 2022

!  AUTHOR:
!    David H. Bailey
!    Lawrence Berkeley National Lab (retired) and University of California, Davis
!    Email: dhbailey@lbl.gov

!  COPYRIGHT AND DISCLAIMER:
!    All software in this package (c) 2022 David H. Bailey.
!    By downloading or using this software you agree to the copyright, disclaimer
!    and license agreement in the accompanying file DISCLAIMER.txt.

!  PURPOSE OF PACKAGE:
!    This package permits one to perform floating-point computations (real and
!    complex) to arbitrarily high numeric precision, by making only relatively
!    minor changes to existing Fortran-90 programs.  All basic arithmetic
!    operations and transcendental functions are supported, together with several
!    special functions.

!    In addition to fast execution times, one key feature of this package is a
!    100% THREAD-SAFE design, which means that user-level applications can be
!    easily converted for parallel execution, say using a threaded parallel
!    environment such as OpenMP.

!  DOCUMENTATION:
!    A detailed description of this package, and instructions for compiling
!    and testing this program on various specific systems are included in the
!    README file accompanying this package, and, in more detail, in the
!    following technical paper:
   
!    David H. Bailey, "MPFUN2020: A new thread-safe arbitrary precision package," 
!    available at http://www.davidhbailey.com/dhbpapers/mpfun2020.pdf.

!  DESCRIPTION OF THIS MODULE (MPFUNA):
!    This module contains some global parameter definitions, including data
!    statements for log(2) and pi.  See documentation for details.

module mpfuna

!----------------------------------------------------------------------------

!   General integer constants:

!   Name     Default   Description
!   mpiknd        8    Kind parameter for long integers (usually 8).
!                      This is set automatically by the selected_int_kind function.
!   mpldb         6    Logical device number for output of error messages.
!   mpmlxm     1111    Precision level in words for usage of mpmulx.
!                      Default 1111 words corresponds to approximately 20,000 digits.
!                      If higher precision is required, call mpinifft or mpinit.
!   mpl2pi     1111    Precision in words of precomputed log(2), pi and egamma below.
!                      Default 1111 words corresponds to approximately 20,000 digits.
!                      If higher precision is required, call mpinitran or mpinit.
!   mpndpw       15    Largest n such that 10^n <= 2^53. See usage in MPFUNC.
!   mpnbt        60    Number of mantissa bits per long integer word in MP numbers.
!   mpnstr     2048    Maximum length of certain input character strings.
!                      See usage in mpinp of module MPFUNC.
!   mpoutl       80    Length of output lines. See usage in mpout of MPFUNC.
!   mprknd        8    Kind parameter for double floats (usually 8).
!                      This is set automatically by the selected_real_kind function.
!   mprknd2      16    Kind parameter for IEEE quad floats (if available; usually 16).
!                      This is set automatically by the selected_real_kind function.
!                      If IEEE quad (128-bit) is not supported on the processor,
!                      then selected_real_kind sets mprknd2 = -1.

!   Long integer constants:
!   Name     Default   Description
!   mpbdx      2^60    2^mpnbt, the radix for MP numbers.

!   Double precision constants:
!   Name     Default   Description
!   mpdpw  Log10(2^60) DP approximation to number of digits per mantissa word.
!   mpexpmx   2^31/60  Largest permissible exponent, corresponding to a maximum
!                      binary exponent of 2^31.
!   mprdfz    1/2^50   "Fuzz" for comparing DP values.
!   mprandx  5501758857861179  Multiplier for pseudorandom number generator.

!   Integer parameters for FFT, log(2), pi and egamma arrays:
!   mplconx    100000  Max size of log(2) and pi arrays. This is large enough to
!                      support precision levels over one million digits.
!   mplfftx   2100000  Max size of FFT arrays. This is large enough to support
!                      precision levels over one million digits.
!   mpnrow         16  Row parameter for FFT.
!   mpnsp1          2  Spacing parameter #1 for FFT.
!   mpnsp2          9  Spacing parameter #2 for FFT.

implicit none
integer, public:: mpiknd, mpiknd2, mpldb, mpmlxm, mpl2pi, mpndpw, mpnbt, mpnstr, &
  mpoutl, mprknd, mprknd2
parameter (mpiknd = selected_int_kind (18), mpldb = 6, mpmlxm = 1111, mpl2pi = 1111, &
  mpndpw = 15, mpnbt = 60, mpnstr = 2048, mpoutl = 80, &
  mprknd = selected_real_kind (15, 307), &
  mprknd2 = selected_real_kind (33, 4931))
integer (mpiknd), public:: mpbdx
parameter (mpbdx = 2_mpiknd**mpnbt)
real (mprknd), public:: mpdpw, mpexpmx, mprdfz, mpb13x, mprandx
parameter (mpdpw = 18.061799739838871713d0, mpexpmx = 2.d0**31 / mpnbt, &
  mprdfz = 0.5d0**50, mprandx = 5501758857861179.d0)
integer, public:: mplconx, mplfftx, mpnrow, mpnsp1, mpnsp2
parameter (mplconx = 100000, mplfftx = 2100000, mpnrow = 16, mpnsp1 = 2, mpnsp2 = 9)

!  These two lines are for debug and timing (for internal use only):

integer, dimension(10):: mpdeb = 0
real (mprknd), dimension(10):: mptm = 0.d0

!  This is FFT data for the extra-high precision routines (for over 20,000 digits).
!  by default). To initialize, call mpinifft or mpinit (in module MPFUNG) at start.

complex (mprknd), public:: mpuu1(mplfftx), mpuu2(mplfftx)

!  The following data arrays contain the values of log(2), pi and egamma to mpl2pi words or
!  roughly 20,000 digits (by default). For higher precision, call mpinitran or mpinit.

integer (mpiknd), public:: mplog2con(0:mplconx-1), mppicon(0:mplconx-1), mpegammacon(0:mplconx-1)
integer, private:: i, i8
parameter (i8 = mpiknd)

!  ALL lines from this point on, except for the "end module" statement at the end,
!  are generated by the compcon.f90 program, available from author.

data (mplog2con(i), i =     0,   199) / &
                1117_i8,                1111_i8,                1111_i8,                  -1_i8, &
  799144290325165978_i8,  849460365225095926_i8,  789272551555017099_i8,  445163727010630570_i8, &
 1126672294557648606_i8,  841214723756325706_i8, 1126376059045602140_i8,   87856769335245609_i8, &
   78421027581422753_i8,  441998077035465690_i8,   77431747161857705_i8,  847716001976624495_i8, &
  724735828797208364_i8, 1047000727954180853_i8, 1007236022851019910_i8,  387205694960493474_i8, &
  982586796677528924_i8,  434585037786799099_i8,  650225694886395477_i8,  117918596040883230_i8, &
 1126277945069789504_i8,   50549047534515675_i8,  619542214527054635_i8,  322514470696668773_i8, &
  664620783330501088_i8,  851697140255516315_i8,  230622882617010631_i8, 1119810899738232653_i8, &
  819679114870534340_i8,  579367960721566869_i8,  420268095221250522_i8, 1022665503503576241_i8, &
   34048807728765745_i8,  819462395931944129_i8,  627356470626432600_i8,  377587242175372002_i8, &
  522106768838125817_i8,  475187583109223216_i8,  790533142032307527_i8,  224296586613789649_i8, &
  155555536112645399_i8,  575281007820180060_i8,  142236722257074943_i8,  956006791660920466_i8, &
  156368997891986466_i8,  109192266016426251_i8,  842577403738103587_i8,  495443378921047737_i8, &
   25916096063257903_i8,  676124210408125699_i8,    9013857688257425_i8,  724049519265889978_i8, &
  563040853642300494_i8,  657662912324494039_i8,  248706839240402313_i8,    8554658232026208_i8, &
  992172116253532354_i8,  658673254866731369_i8,  494591850600231810_i8,  503735489371114161_i8, &
   79863948294375794_i8,  923673311921914476_i8,  320328893528628229_i8, 1112244984639079854_i8, &
  931320635882439481_i8,  977114157202401929_i8,  139338565954472227_i8,  526383901974710490_i8, &
  782931846310331300_i8,  701732238205853066_i8,   63683354286319583_i8,  673434243579618325_i8, &
  175548377669993545_i8, 1146437748375563034_i8,  448790188173634227_i8,  222398369492404602_i8, &
  260337464341295972_i8, 1106630893137309678_i8,  495542257580167279_i8,  796140836195131404_i8, &
  384125646372631712_i8,    3563472316330036_i8,  129446520001227935_i8,  669307132983046785_i8, &
  360821498254194255_i8,  123711642274902542_i8,  216024568108898111_i8,  778102238170547600_i8, &
  222184626363311026_i8,  144718806408174580_i8, 1100108707831828236_i8,  979373091756646605_i8, &
  818752698118121613_i8,  398099411270273647_i8, 1031791399515909930_i8,  975666882559521296_i8, &
  826302772131868796_i8,  901957094641068366_i8,  985468267140460917_i8,  140636129575982693_i8, &
  582443381339988004_i8,  436965351657920632_i8,  329690268516097450_i8,  361755822954707778_i8, &
  836048778640073208_i8,  427683480088008861_i8,  190472261838896532_i8,   24640005490158421_i8, &
  182284053086034962_i8,  676578978331011955_i8, 1034521786910346870_i8,  166374410579214955_i8, &
  468517924915632539_i8,  463238016996657368_i8,  914123998354776640_i8,   57079355793199360_i8, &
  247591949805732633_i8,  369869227079445107_i8,  828287238903563655_i8,  139465334086976052_i8, &
  231199710005878699_i8,  293935834227235306_i8, 1010239010349777194_i8,  588109827225105535_i8, &
  590056800218899881_i8,   88342550679400581_i8,  197664220934771921_i8,  973390870419564439_i8, &
 1022354071638101640_i8,  327223931981822834_i8,  673719157925286936_i8,  499260709773167827_i8, &
  881420100402606352_i8,  749712803814218956_i8,  267805591355837971_i8,  342981175418154092_i8, &
   87374448893336260_i8,   35425629321345233_i8, 1067200725895442892_i8,  651708134106010702_i8, &
  468020985336532970_i8,  548746280850768044_i8,  424306061864281399_i8,  562675447305845169_i8, &
  799083858332991258_i8, 1034251353089834270_i8,  506993838410332019_i8,  479369101179680390_i8, &
  621590972165683106_i8,  863492129211160691_i8, 1084316836670123534_i8,  688503277056753841_i8, &
  789791707159317783_i8,  790387065527662561_i8,  837134085022055354_i8,   11277928609406305_i8, &
  518861726845039286_i8,  440464703345610292_i8,  121718278379253877_i8,  759832941642890638_i8, &
 1076920608631819102_i8,  717772455104956408_i8,  568984710648663741_i8,  205477930870397675_i8, &
  269784892369263667_i8,  849183475259682338_i8,  616866334384698888_i8,  692800234392969398_i8, &
 1152230549462934572_i8,  500503152127649118_i8,  126245498771068966_i8,  890929255574027784_i8, &
 1060168635810577430_i8,  595077510428262677_i8,  731890091658661833_i8,  986954251979502786_i8, &
  527834251376216053_i8,  416866295771433412_i8,  413817092072915606_i8,  679518426634438959_i8, &
  367071260004420686_i8, 1112439051266371949_i8, 1067902753019227613_i8,   88389281379550990_i8, &
  512312274980148782_i8,   77386272335476273_i8,  150726669523121823_i8,  614515173270362700_i8, &
  692597806892870716_i8, 1023928212596610006_i8,  481491362894488537_i8, 1044650816900609030_i8, &
  625698959695448248_i8,  575620053285159746_i8,  983938581084548586_i8,  427350812476028405_i8/
data (mplog2con(i), i =   200,   399) / &
   26214601062588532_i8,  155403891005965895_i8,  257567160581744906_i8,  248812513404632388_i8, &
  322811653660106279_i8,   98301930540390155_i8,   95365878972252892_i8,  318320877913390204_i8, &
  231009522683768740_i8,  362462804834654824_i8,   51650025446364022_i8,  941349327986594338_i8, &
  858214689172523519_i8,   97009468687083271_i8,  533457405689974006_i8,  961566524321172902_i8, &
  748893129930498356_i8,  739359652063325213_i8,  376186840024265488_i8,   81685434742660315_i8, &
   18552275975468577_i8,  759513350041907521_i8, 1107918750620629381_i8,  482623678831139631_i8, &
  313587492061892516_i8,  970119875325489495_i8, 1152029418603015670_i8,  292752718313153111_i8, &
  729652919649859847_i8,   59121243270169511_i8,  883012910660207885_i8,  911883276820725257_i8, &
 1108060247203002448_i8,  777499776365958319_i8,  506579131957850575_i8,  590390476447894226_i8, &
  919038613562131972_i8,  763797749262680806_i8,  607116556032205078_i8,  618832429999095046_i8, &
  854095627383029666_i8,  147420236702569455_i8,  783122135413127828_i8,  287429999575561501_i8, &
  434511294326224078_i8,  663224757235775484_i8,  511524473030045632_i8,  642446440584494050_i8, &
  837936411221611485_i8,  753614697706028886_i8,  388231504997748244_i8, 1145159457782254941_i8, &
  569945831225802635_i8,  886766057663486355_i8,  975475112102166462_i8, 1151236252682073041_i8, &
 1148097606167348226_i8,  394957217600550809_i8,  771462837894568267_i8,  998122784548308168_i8, &
  503381597953737174_i8,  322833980530501374_i8,   87865924882473497_i8,  714122648467357109_i8, &
  752673846782047851_i8,  250362701470943663_i8,  205040300332387289_i8,  977514481981426545_i8, &
 1135119110138554444_i8,  230249746973775135_i8,  443104546631608384_i8,  245356068942335423_i8, &
 1030457727680559326_i8,  192125873291591777_i8,  283210844891684472_i8,  555282640076409001_i8, &
  998049996142846735_i8,  591085771260265611_i8, 1000408297633808930_i8,  108251260765057132_i8, &
  912112999405025082_i8,  371580983923799882_i8,  461139158578143836_i8,  382447360157113668_i8, &
  128210220471918635_i8,  527076485125328577_i8,  805699606227145128_i8, 1116279669424834780_i8, &
 1048040573894917156_i8,  143576057199832021_i8,  519513581632878400_i8,  663887436479339473_i8, &
  238951085900059841_i8,  982094536536151870_i8,  642992019531266513_i8,  500695602980965738_i8, &
  923792999635687764_i8,  695738776817694694_i8,  559009904195048011_i8,  744630899780482644_i8, &
  716978105369268336_i8,  334185617048924857_i8,  897074423348081097_i8,   88807033510183842_i8, &
  443586168665421195_i8,  173781834081243802_i8, 1113212896894046626_i8,   27129351640591175_i8, &
 1004081622300772378_i8,  302418987647375213_i8,  262044137353239909_i8,  302105098921156801_i8, &
 1121407493370256038_i8,  673411397790718087_i8,  832335897494446633_i8,  800897952968847766_i8, &
 1034238603232108545_i8, 1096251132131054419_i8, 1119274019573266152_i8,   66405056886100240_i8, &
  292019770913004824_i8, 1059126497750198958_i8,  976629425460808316_i8,  581380674599560706_i8, &
  620550942476223870_i8,   88133070316701006_i8,  411392517627898966_i8,  441577778036813677_i8, &
  867189274265345459_i8,  759147152494146751_i8,  575054746618095041_i8,  901679738337181063_i8, &
  808253674359980128_i8,  792111553053836419_i8,  857333890882998476_i8,  316899465187571751_i8, &
  677616162445074362_i8, 1060780114976855105_i8,  948117946844501581_i8,  744702827442721304_i8, &
  571866882866149613_i8,  803644270804915261_i8,  305782621818284514_i8,  838164803311495193_i8, &
   76151488926609854_i8,  585522141654743831_i8,   23008054201576536_i8,  683442384478426519_i8, &
 1074787843321260223_i8, 1029376022591830482_i8,  766420856548136942_i8,  131778701675867495_i8, &
  278548947942778515_i8,  666628162075145654_i8,  874713406086314666_i8,  349223771627071413_i8, &
  863589001508057903_i8,  836916824891101261_i8,  507350599049757394_i8,  529082725265274023_i8, &
  488608319470447175_i8,  639663530610273294_i8, 1025519810974580434_i8,  607294123120628110_i8, &
  428460088289282263_i8, 1002420650540683836_i8, 1106180566704768674_i8,  558457124115438900_i8, &
  366389169228121681_i8, 1147442990070156565_i8, 1082949516233001151_i8,  986794671236436018_i8, &
  278667451809049587_i8, 1145981525557924905_i8,  981544739076106474_i8,  873732650238978614_i8, &
  832928290735410628_i8,  231619633199686161_i8,   61933911224412632_i8,  402116636713115237_i8, &
  423729562159488429_i8,  501843926785311192_i8,  287033390816431576_i8,  593396548481295684_i8, &
  772218935421849099_i8,  276532313064683585_i8,  735268998693803397_i8,  700729010767290589_i8, &
 1085540102245623045_i8,  958624864118475035_i8,  249245384339645008_i8, 1070855605738532307_i8, &
  901005772729046013_i8, 1024208502531112131_i8,  807539004513036781_i8,  809172551059996491_i8, &
  810315367126421240_i8, 1138624523581978994_i8,  877927850664600021_i8,  144027399777300786_i8/
data (mplog2con(i), i =   400,   599) / &
  436526319822532119_i8,  668467541211163834_i8,  591122951432388562_i8,  394235439063829825_i8, &
  540233151069418882_i8,  209615697160396555_i8, 1005681268146132044_i8,  687222575101143666_i8, &
  451412544336235979_i8,  658864515524322831_i8,  376552379867535592_i8,  415581362118889335_i8, &
  631151753035475428_i8,  892195505652087600_i8,  450266715286191998_i8,   92477002138264229_i8, &
  421316017955593964_i8,  257222533777835219_i8, 1055621578801905721_i8,  912369946434415798_i8, &
  169343746217759468_i8,  628221467855807740_i8, 1088954373021956749_i8,  888583260959617556_i8, &
  232422717944375095_i8,  570456111554037021_i8,  102975415716891560_i8,  768522359655049862_i8, &
  905038678507803004_i8,  408026425344761149_i8,  742177891914447247_i8,  336616977292951984_i8, &
  959202627277445654_i8,  677760873048061818_i8,  975052682095487499_i8,  679627407719411960_i8, &
  586433832718688904_i8,  156616636098739056_i8,   16565359989183682_i8, 1137052670363971832_i8, &
  596773424405546043_i8,  517530388650724475_i8,  709570640198686087_i8,  502148477400685260_i8, &
  431205268083860841_i8,  466390995821334447_i8, 1004921374603432691_i8,  939637548149927839_i8, &
 1119451909969934809_i8,  184118174201989108_i8,  487998085446249452_i8,  854513241285781248_i8, &
 1022111897304564526_i8,  660148626420111644_i8, 1000257180853925634_i8, 1050362794200386847_i8, &
    2372508557559026_i8,  756765172108950920_i8,  719995798411006350_i8,  440921952231799691_i8, &
 1031634920328574259_i8,  764158649254010567_i8,  436510647072944019_i8,  487342854280380908_i8, &
   25353863464416841_i8,  495411585321713556_i8,   53726128399531960_i8,  672623234073483761_i8, &
   25409158502286721_i8, 1101328548529495890_i8,  468786814663215870_i8,  984409316581227395_i8, &
  562109035928892940_i8,  132923348378120938_i8,   10529244561450124_i8,  584075375476198028_i8, &
  435441147016303507_i8, 1121389413910576846_i8, 1119116310999176187_i8,  580812415191731375_i8, &
  607145523617859515_i8,  138040645243900528_i8,  993952911971738242_i8,  586315428164924197_i8, &
  693995277950933242_i8,  450345089478105356_i8,  778527095503729870_i8,  765382812727126450_i8, &
  272815728916914978_i8,  110271630373740955_i8,  104310950927544874_i8, 1025406195559591999_i8, &
  908377393288874436_i8,  949114445089141098_i8,  744104746054708323_i8,  849571492676452013_i8, &
  792063177787883245_i8,   97980040087077128_i8,  366600859212710196_i8, 1046827633420619518_i8, &
  217426565891488071_i8,  164267268510338615_i8,  393241344941049551_i8,  248457607393726076_i8, &
  718036578751400877_i8,  309273214641235128_i8,  346985850135213665_i8, 1000764186787798137_i8, &
  844130522682825875_i8,  374151902974448954_i8,  607758494955793995_i8,  532672319999474291_i8, &
  613969964055402732_i8,  780410419194198488_i8,  394895139131331966_i8,  165455233759011797_i8, &
    6322931772207497_i8,   86161037070584406_i8,  791443389714935878_i8,  835759646653365969_i8, &
  687059935766086720_i8,   14214845501010438_i8,  564352774506296125_i8,  656139210352258808_i8, &
  813986333089368255_i8,  187942176056140984_i8,   55182364551670509_i8,  189085997088766303_i8, &
  769961772185477576_i8,  452568376836315185_i8, 1032483914756410033_i8,  733074542523761510_i8, &
  725078382924804854_i8,  370364211687393561_i8,  211518325664242562_i8,  198402155053317542_i8, &
  701558338072560331_i8,  685156488055193556_i8,  972894316377842164_i8,  618715301818574266_i8, &
 1120461702831103893_i8,  266035263849868872_i8,  695159837979264076_i8,  580660299994441057_i8, &
   66777801238183396_i8,  808242202788623316_i8, 1131958431733874673_i8,  143428505846952154_i8, &
  478658011657139736_i8,  424286141989330875_i8,   97944871024137655_i8,  127584227459937021_i8, &
 1026980244582044584_i8,   93529499555819729_i8,  786072881728426537_i8,  243198104306627423_i8, &
   54461428243013698_i8,  341821771035463176_i8,  328610091866341975_i8,  589596999450239151_i8, &
   22268131370076300_i8, 1125497851362970258_i8,  338309767149066446_i8,   48501308837176444_i8, &
  506783924216248724_i8,  550494901033541943_i8,  853224896050219021_i8,  540896285386052939_i8, &
 1087963595273741451_i8,  590236422416936651_i8,  428199315726494527_i8, 1017913446427102704_i8, &
   11627268202307574_i8,   12411272720587191_i8,  151571015692077823_i8,  801805729255844690_i8, &
 1053129572400431340_i8,  898149488598553006_i8,   38812608295917889_i8,  217051013754214696_i8, &
 1027877295994496961_i8,  243099290769625466_i8,  613955609889484599_i8,  353870262657925549_i8, &
  637141720733912176_i8, 1127408345606387682_i8,  572192062258313139_i8,  936847025720831096_i8, &
  741596068164486248_i8,  566216362278956981_i8,  100840248057817976_i8,  737232715384065277_i8, &
 1082245789529037312_i8,  788517729010872412_i8, 1031957427233188981_i8, 1104677670128566892_i8, &
  432365578112025669_i8, 1087453812475234390_i8,  842879314247712024_i8, 1091855776801052064_i8/
data (mplog2con(i), i =   600,   799) / &
  278865056457498733_i8,  274452313453978557_i8,  908556800533554594_i8,  577869998717774069_i8, &
  670991146488849126_i8, 1149310286936094747_i8,  306699532648224706_i8,  457514022108522018_i8, &
  406707506328130099_i8,  871051767037809673_i8,  295783931878198179_i8,  322158716553114590_i8, &
  810383792479010363_i8,  583197357626951498_i8,  124697906935340833_i8, 1051837863810602196_i8, &
  772112765990823686_i8,  211747207089338424_i8,  799169051668571380_i8, 1147020419962261891_i8, &
  456292243478347382_i8,  556793759222586816_i8,  256638549248309571_i8,  136321905145156104_i8, &
  734776624627421328_i8,  776166734497884968_i8,  695577266205460858_i8,  883556684626115256_i8, &
  983723563201364370_i8,  776786197748668391_i8,  475552184055096133_i8,  844266604857406075_i8, &
  121615283512287317_i8,    9067581013474256_i8, 1021560708271259533_i8,  906274065814214948_i8, &
 1132245209448321733_i8,  460342749845500015_i8,  780415152079123787_i8,  905996371999437994_i8, &
  383819064556268321_i8,  196051615838505789_i8,  188741022513163602_i8,  227986482207525482_i8, &
  407229126723239000_i8,  190085214224497956_i8,  277296367059184976_i8,  355820437457341298_i8, &
 1078854378499317804_i8,   32020025387782691_i8,  640720685190603086_i8,  852734663037663378_i8, &
  264464739082922377_i8,  335024594286679694_i8, 1135663454385116605_i8, 1120649339250429251_i8, &
  394209779522642688_i8,    2590995859022910_i8,  682752152747476324_i8,   94445426907114725_i8, &
  664175841519509107_i8,  200099227187547097_i8,  980720931651783828_i8,  175110786622650762_i8, &
  326249008465240636_i8,  222289956251610900_i8,   84401765198866404_i8,   43951738232093651_i8, &
 1061998581558766947_i8,  804016896777570374_i8,  553987896409405118_i8,  328381755879513320_i8, &
  168077324838983330_i8,  962664752752360065_i8,  400567386176322048_i8,  872842448194885666_i8, &
  579472432376865602_i8,  191824016414278687_i8,   44792944137807009_i8,  557072048817284581_i8, &
  464780468776130630_i8,  953713177036138577_i8,  547498276706121232_i8,  492659725118967205_i8, &
  729853912709027694_i8,  121906712729630341_i8,  846479797716944612_i8,  344666898443009521_i8, &
  344359097202429800_i8,  939005880601151541_i8,  739469902012256961_i8,  522699315080500473_i8, &
  928611579704408774_i8,   29824290460323223_i8,  875823776529951743_i8,  923035084868059011_i8, &
  309221547396307487_i8,  551955627778925301_i8, 1144949620625652722_i8,  208885703961551051_i8, &
  857874732888983563_i8,  142455790380519086_i8,  796075857165951926_i8,  186421529818299714_i8, &
  986930304450790647_i8,  875600270974391552_i8, 1013234744964277088_i8,  563065323890303076_i8, &
  803630673386550924_i8,  414190818950026776_i8,  129105646808951793_i8,  300998170897486172_i8, &
  473454667902653769_i8,  873512649259161579_i8,   98472778980610450_i8,  136551994856514226_i8, &
 1077787456836457044_i8,  840612906558582321_i8, 1099970926062002697_i8,  752497634121942619_i8, &
  151512647715585905_i8,  342868545578500791_i8,  578889956118752508_i8,  448735422218391621_i8, &
  672062101866778817_i8,  775940557852584381_i8,     243985095924355_i8,  271365311073457276_i8, &
 1126841083685055294_i8,  132604189951045671_i8,  884730995621884774_i8,  980685837546167370_i8, &
  497367762513226619_i8,  584582987052762645_i8, 1104022329469484618_i8,  697139211828998291_i8, &
  415963849299775980_i8,  240621556900763402_i8,  832452481724583121_i8, 1107969795684588140_i8, &
  800041755863870066_i8,  100851287044377902_i8,  839058348490036754_i8,  493920407171369266_i8, &
  321652836031992975_i8,  753445085031930560_i8,  961245074392204540_i8,  299400470291371394_i8, &
  646378631354413115_i8,  243818903187757550_i8, 1022733408199555141_i8,  139321989410550371_i8, &
  548597010519710044_i8,   83458605421088928_i8,  501962253553281099_i8,  464643749909362817_i8, &
  195257349814776012_i8, 1141292733263619832_i8,  194278865892404084_i8,  241466388992020488_i8, &
  777347073769003549_i8,  514876560340780717_i8,  345470029516725675_i8,  553372236299236517_i8, &
  982422699855201039_i8, 1005520980705024510_i8,  510778287126193029_i8,  225791943704232171_i8, &
  731243211867607504_i8,  278240161079565852_i8,  967511878183205559_i8,  988808584380813902_i8, &
  789099488668750831_i8,  915425111915078982_i8,  488862429648177400_i8,  823450492076598646_i8, &
 1023499952444809049_i8,   24927986448791912_i8,  287862703120185408_i8,  828608263965900141_i8, &
  242140655783397369_i8,  237589385998706936_i8,  943808472644620496_i8,  567924766317904788_i8, &
  193435228639052855_i8,   22499843949503154_i8,  360703823095636586_i8,   85863009663757278_i8, &
  660109162120644625_i8,  845928119477630086_i8,  898001062188705721_i8,  443980487767824843_i8, &
  954148268409556975_i8,  999864851324966066_i8,  517177309607441607_i8,  660794004363599433_i8, &
  488234593502099210_i8,  638320916434820525_i8,  243359375222367005_i8, 1020713578498329900_i8/
data (mplog2con(i), i =   800,   999) / &
  171542867853781370_i8,  107107949954094897_i8,  774688289940185318_i8,  232539131488437753_i8, &
  488058626181754703_i8,   30903178929158685_i8,  188808334543967560_i8,  211254432868961451_i8, &
   78850033006999759_i8,  798568116846010569_i8,  848939684751205301_i8,  566799850712825947_i8, &
 1048167811024057127_i8,  282948432002862356_i8,  456461113592093132_i8, 1040177238368624989_i8, &
  517450413486254840_i8,  806626212448774680_i8,  998910289981461359_i8,  470453719173931346_i8, &
  269087033452990026_i8,   79955433853542890_i8, 1124633564761178700_i8,  856853245839436641_i8, &
  387174561729283646_i8, 1052650223591992925_i8,  280751504714819633_i8,   29656364971427369_i8, &
 1006082156511733776_i8,  829128948265754398_i8, 1059676261406298182_i8,  228619449878404868_i8, &
  809897153290506183_i8, 1090339931415742412_i8,  996607302723064978_i8,  538870134414869278_i8, &
  107585297277568183_i8,  202666001776158248_i8,  419409826462444037_i8,  840811152998489919_i8, &
  264903289087932283_i8,  688078115437172311_i8,  612726383633207051_i8,  929374056782094999_i8, &
   61607043280589062_i8,  122169072554909443_i8,  763125301038599055_i8,  301081401927897956_i8, &
  947058606381851692_i8,  460238178716984995_i8,  265878786938840815_i8, 1069223716622168918_i8, &
  113543438453979710_i8,  217833552319080273_i8,   23836046264100017_i8,  879377680201552019_i8, &
 1033850117437227616_i8, 1008884643383989693_i8,  582788836566860337_i8,  870762109129573952_i8, &
  150990451617713430_i8, 1083582584353257121_i8,  274048091708929711_i8,  624890733022221610_i8, &
  629386208685503491_i8,   17161929256431812_i8,  894223605435649381_i8,  531668758695752063_i8, &
  253911493609370662_i8, 1134337565022246839_i8,  421619976780838838_i8, 1048120104303492877_i8, &
  955832471347784992_i8,  393451459371335945_i8,  659836388821532813_i8,  202550901643322383_i8, &
  113411407396430508_i8,  264815521189254049_i8,  422447937075123938_i8,  369708236503305351_i8, &
  691364732958192854_i8,  106560166810177108_i8,  875549310068077107_i8,  644063540984015438_i8, &
  821864291168628043_i8,  755402110936009514_i8,  608148612990486740_i8, 1032248994184851768_i8, &
  983316556886450464_i8,  724554986426719728_i8,  747747654472932205_i8,  167097846868702375_i8, &
  315178218777542199_i8, 1015169444386770642_i8, 1058419236100997368_i8,  949458617632420692_i8, &
  211775190891291587_i8,  410560258396857851_i8,  497611518673367437_i8,  693491450387497196_i8, &
   61978781257529642_i8,  842396480052797423_i8,   21607201294239378_i8,  688886848235344630_i8, &
 1109432658327892577_i8,  805986620017780112_i8,  823543484053637107_i8, 1134566853605974278_i8, &
   83435206945411286_i8,  751313197793774077_i8,  406314023429654675_i8,   25233395010318695_i8, &
  756962578943244795_i8,  537098983847573656_i8,  596733154586351762_i8,  462223637778679059_i8, &
  633617075101491551_i8,  432787930512540973_i8,  423288589638437965_i8, 1042658474804511529_i8, &
  855417183730115965_i8, 1074092783412956028_i8,   97831489848130843_i8,  406570034206291979_i8, &
  737531051146394151_i8,  865991417462274661_i8,   90258971504800752_i8,  202582896335699879_i8, &
  414340105538112018_i8,  467628253845031205_i8, 1048114869176353655_i8,  100261917520860596_i8, &
  418409569780185631_i8,  109308988914969830_i8,  309097446051190109_i8,  518273349308334253_i8, &
  535038700955613132_i8,  174851455450805754_i8,   92975413975980655_i8,  873811500227205444_i8, &
  934510158897864617_i8,  747636797957241269_i8,  756714198212757108_i8,  543822488904404678_i8, &
  505608277523868598_i8,  601922220038810633_i8,  854151317755363235_i8,  292530720454406209_i8, &
  261200008100242161_i8, 1100418728661175114_i8,  238416850270270577_i8,  847586693533435459_i8, &
  492209755400529064_i8,  913861962008259670_i8,  788663778219619282_i8,  604829301272092109_i8, &
  345486168526580491_i8,  842586687750480812_i8,  453081855055969393_i8,   88284284043064152_i8, &
  724296532201234089_i8,  973278548817724308_i8,  576820060600330947_i8,  623983914301568406_i8, &
  220752079570261873_i8,  706101464645326042_i8,   39208276311132605_i8, 1106578302091867316_i8, &
  527381544245534032_i8,  787270277233968762_i8,  925728131504908658_i8,  480794412548574332_i8, &
  640807268486495588_i8,  733165658843001879_i8,  388666546765834172_i8,  580155276576284938_i8, &
  248629367540737029_i8,  147696527560579820_i8,  172236573393900306_i8, 1116962164969714823_i8, &
  656112047776515219_i8,  860077860470590047_i8,  267034879451351981_i8,  734505304602729064_i8, &
  334984534101030130_i8,  379994277910803963_i8,  118371934974875025_i8,  395972233855319102_i8, &
  483343081821137362_i8,  695170314802749080_i8,  475483276471859807_i8,  457046897738984227_i8, &
  646511902967665049_i8,   23419384342660522_i8, 1148251973969710269_i8,  997084996784238994_i8, &
  182129975172660425_i8,   16107552625356663_i8,  305769598179078383_i8,  285372649627709787_i8/
data (mplog2con(i), i =  1000,  1116) / &
  426154842883288599_i8,  971820862552434137_i8,  805034237090529991_i8,  129381793112173426_i8, &
  796349097178852575_i8, 1095914385410367491_i8,   60183289318960471_i8,  584599920292003143_i8, &
  891623033889996477_i8,  335818849308490003_i8,  338611762659913393_i8, 1100499991732860217_i8, &
  717570056436016399_i8,  705222587167423071_i8,  286517318764576461_i8, 1031625786751779509_i8, &
  725130176905804763_i8,  958299491271527994_i8,  393335852370618453_i8,  712511833084678755_i8, &
  995715193796251867_i8, 1130301873578568392_i8,  270196921674700807_i8,  599909883133692107_i8, &
  774578491820431175_i8, 1108059265840577363_i8,  366249482988655571_i8, 1109218225939118509_i8, &
  646926677756056721_i8,  271098737163307748_i8,  850662509776491758_i8,  723107737018722679_i8, &
  197335391718316535_i8, 1058512337403515113_i8,  667961857345747927_i8,  932218810215668076_i8, &
  689825524198451109_i8,  347770987401131366_i8,  811685514724324328_i8,   24554820806102562_i8, &
  978119655449061923_i8,  750862400882654319_i8,  898565190186309310_i8,  488630659903647688_i8, &
  338426341570688862_i8,  548247867971520812_i8,  305448011229652694_i8,  604965641380790330_i8, &
 1091814844674968957_i8, 1135761340388512854_i8,  291605092188706489_i8,  724249218186247352_i8, &
   75812089031399671_i8,  571016455582880565_i8,  606860802694100487_i8,  372234225297475155_i8, &
 1097366722261076015_i8, 1031478345924192313_i8, 1022316987518395207_i8,    4466821150800125_i8, &
  990319123187449648_i8,  725423144304712913_i8,  981258478983626423_i8,  758262587395703761_i8, &
 1025072375997053091_i8,  789317478820524791_i8,  883841627203700306_i8,  545042391596686992_i8, &
 1094203306398013013_i8,  219085193191197938_i8,  328054539967724316_i8, 1121894228490578130_i8, &
  542660567945635232_i8,  854025998308892431_i8,  348473564494718249_i8,  499893383199186813_i8, &
   77505978302706138_i8,  291209354648696484_i8, 1041548175314127155_i8,  710376159574114493_i8, &
  742459491848682217_i8,  866912019902139358_i8,  749873916030853074_i8,  265506355397247338_i8, &
  812238398374144870_i8,   32877983561235972_i8,  289572229816589031_i8,  673341386514222320_i8, &
  441583844367026154_i8,  321316735962878817_i8,    5446928668731077_i8,  842593796470306992_i8, &
  131917728081982580_i8,  620962259354022480_i8,  126931996390897249_i8,  725451423698915638_i8, &
  910983165055558525_i8,  304653650992348508_i8,  787842812933121661_i8,  926219265248741290_i8, &
  832322490588383093_i8,  363036159374422479_i8,  803568520393121849_i8,  855832571969749538_i8, &
  137003653649042314_i8,   18111990899473286_i8,  645309673747536544_i8,   56841936167960499_i8, &
  291710102143335432_i8, 1028035077033195773_i8,   50934365469588043_i8, 1056350394205772163_i8, &
  351398385142814178_i8,  520906985190618220_i8,  151310990670167219_i8,                   0_i8, &
                   0_i8/
data (mppicon(i), i =     0,   199) / &
                1117_i8,                1111_i8,                1111_i8,                   0_i8, &
                   3_i8,  163245215218020493_i8,  221548887939444851_i8,  309130526715976179_i8, &
  130613387059588174_i8,  488803641660957581_i8,    5479683901017908_i8, 1049557292833479548_i8, &
  683510154637137333_i8,  816401202565705053_i8,  687214537561157899_i8,  750286923632934871_i8, &
  205987021816576225_i8,  792306577480182695_i8,  905300427237464356_i8,  727776071756730224_i8, &
  577009281997405206_i8,  447705799768503526_i8,  694778093217551165_i8,  567692521958222059_i8, &
  457240207437627925_i8,  337461787920292901_i8,  745826895745333546_i8, 1091561318455640626_i8, &
  603629619299580184_i8,  832518932776066507_i8,   27085472759848462_i8,  629098671642603863_i8, &
  558935848182577327_i8,  215510146066288229_i8,  370970457235231831_i8,  326939946776609797_i8, &
  417209043630362806_i8,  814243075048349324_i8, 1054216725227926249_i8,  665194507147818747_i8, &
  876561085637686296_i8,  225018240481454520_i8,  545813720412599148_i8,  165777524090288146_i8, &
  618548370491787416_i8,  483217334367682177_i8,  821388106713389065_i8,  923152275941394378_i8, &
  891815996563844996_i8,  420486886110993858_i8,  442200398449313827_i8,  618092786094885968_i8, &
 1111667774711939641_i8,  207363804899066368_i8,  318006183075913627_i8,  423932541556190870_i8, &
  908151578729491411_i8,  616718980149489029_i8,  301574693774108843_i8,  365700002358539969_i8, &
  250578582835556432_i8,  571872118935852625_i8,  952985216981846617_i8,  281514769188966120_i8, &
  439458784368426372_i8,  746528506655796742_i8, 1114034551826445120_i8,  118300808647715428_i8, &
 1068384875410585350_i8,  126080960309080099_i8,  952459468666309138_i8,  328112955742002972_i8, &
   43636676379639961_i8,  123763628479188846_i8,  639220755729488566_i8,  546276656148253648_i8, &
  342280970862940086_i8,  291034183514835436_i8,  151268767012813679_i8,  789228094498091931_i8, &
  210327034942942716_i8,  365347592144604360_i8,   91554887430375870_i8, 1025976624634703830_i8, &
  436611699830574003_i8,  868273910521300800_i8, 1139986159296435195_i8,  987792021002495491_i8, &
  147165924834033708_i8,  515546132858793905_i8, 1126969244813752539_i8,  225795174353169919_i8, &
  964169533966261960_i8,  779216377417554783_i8,  791828173830697339_i8,  325349948703499717_i8, &
  557261559359871623_i8,  388134139240234324_i8,  191673637953061804_i8,  464764015041926966_i8, &
  674176209240348872_i8, 1017807150032814362_i8,    4778742094045571_i8,  829981519211846941_i8, &
  241956690100598520_i8,  312539654364316863_i8,  835295555666238116_i8,  916450871061066780_i8, &
 1073201314967636698_i8,  245293073184582127_i8, 1021174859005549530_i8,  349848146367392472_i8, &
 1040731271587745934_i8,  944923576184473827_i8,  890357640240347023_i8, 1111878655948273224_i8, &
  614943739001827356_i8,  358834321438080049_i8,  923748035026200769_i8,  779952807247339745_i8, &
  538156469497793282_i8,  142459710325455702_i8, 1113770200545875662_i8,  620979545570606607_i8, &
  942343825727437697_i8,  948810973079534118_i8,  468538699673750643_i8,   90654326887705298_i8, &
   28560596123699028_i8,  301564951776788156_i8,  986020566887145686_i8,  293223511351616656_i8, &
   10430161118933854_i8,  154952024968629770_i8, 1091107195437713849_i8,  136610166817005050_i8, &
  480329081725507930_i8,  376035209373163566_i8,  412578007950259810_i8,  674689545354188420_i8, &
 1041257584893832650_i8,  554306231371894789_i8,  187553770414411607_i8,  287044812409058826_i8, &
  321587250192189626_i8,  503089730398846761_i8,  498891487096411574_i8,  447635503846497135_i8, &
 1095506307798357604_i8,  376898111334381818_i8,   74514210531214599_i8,  478709388540599090_i8, &
  667899940469392409_i8,  171764430523794586_i8,  567310191039658127_i8, 1070491593718941462_i8, &
  691891878188307052_i8,  876823062741475370_i8,  393230765185325331_i8,  292243059471807817_i8, &
  619908240383503247_i8, 1030596023304415773_i8,  188034011416819021_i8,  203663015593499668_i8, &
  926932860427889446_i8,  448159412085976261_i8, 1011454401153252079_i8,  906289439368984231_i8, &
   45432366818415264_i8, 1020183486342912677_i8,    2031064390573183_i8, 1002866518089027269_i8, &
  510862482797681207_i8, 1082272113827627504_i8,  288794099193875701_i8,  118161591024900151_i8, &
  745587670108721553_i8,   89957413876099395_i8,  164226505403466042_i8, 1035361805052784076_i8, &
  627537691398102439_i8,  762311290272940080_i8, 1075455169109718302_i8,  296713713305763488_i8, &
 1022091168921491518_i8,  807011733229241590_i8,  988830867127075830_i8,   45119502442113282_i8, &
  330869958218723442_i8,  849764824281360241_i8, 1142105643364354990_i8,   84949316114141479_i8, &
  152891951348469786_i8,  999352682484889509_i8,  596472362293663676_i8,  718794519288993998_i8, &
  899290813149027834_i8,  446019324575370301_i8,  211418598108960217_i8,   39921290235235371_i8/
data (mppicon(i), i =   200,   399) / &
  726788873862195813_i8,  884136031320809779_i8,  784995716227964432_i8,  222054448217161061_i8, &
 1085677138754230001_i8,  971155150171345283_i8,  817725098599550950_i8, 1070026160813501355_i8, &
 1093281886101210851_i8,  312734842145675360_i8, 1035276347048113015_i8,  143677328572051362_i8, &
  677190008698248463_i8,  577560939938081922_i8,  373320247313961139_i8,  480866260400819758_i8, &
  754535440346979850_i8,  206971173125911393_i8,  948919325033893657_i8,  875833601340108943_i8, &
  509771759670394478_i8, 1057274947006350425_i8,  407148917456731959_i8,  941279603527004973_i8, &
 1004230614513444184_i8,  150965741334572780_i8, 1055267121758812970_i8, 1078571247209740353_i8, &
  369776928024387952_i8, 1066742295767413073_i8,    5573047304806845_i8,  675596362291257398_i8, &
  274637609371716720_i8,  730600574065622202_i8,  783086401075199072_i8,  442138337876495035_i8, &
  212659217084653231_i8,  115931460912952987_i8,  677047079540910170_i8,  226238447255493102_i8, &
  708683767450409812_i8,   49988703351314783_i8,  547607790105766136_i8,  250101140378802033_i8, &
  138929200125448833_i8,   63992137707839997_i8,  474811862802258106_i8,  690598135051956775_i8, &
  172152201001507522_i8,  317904937376724274_i8, 1022410092215592045_i8,  847541519710629619_i8, &
  344243806291946849_i8, 1074042296520188237_i8,  713931281892773981_i8,   90639208443829998_i8, &
  195523587261258604_i8,  356171040914476148_i8,  212868678448559461_i8,  356824320067113053_i8, &
  585038723373419975_i8, 1028145036595303973_i8,   58373767874448678_i8,  319694585981574267_i8, &
  478365714219231691_i8,  524990930598540649_i8,  194364009170796717_i8,  113867422509504322_i8, &
  252779923729714474_i8,   55555430459039555_i8, 1018727692615512816_i8,  660016179483338238_i8, &
  932645492131331294_i8,  249277698126775304_i8,  428589419758513830_i8,   33786075147471342_i8, &
  643971132962740021_i8,  762967038553538668_i8,  169453072701983690_i8,  993576189139104851_i8, &
  325664166075764254_i8,  715119851185451269_i8,  398794410895768035_i8,  465053889218168407_i8, &
   89571543984175262_i8, 1087302590553907742_i8,  544847857231258527_i8,  197516521671398357_i8, &
  750012620818871884_i8,  171639565760807953_i8,  369567255670093756_i8, 1100763891159271053_i8, &
  290522244573099114_i8,  304975162816595482_i8, 1089371889376633925_i8,  315606106708375736_i8, &
  594560457388168885_i8,  656108340838741610_i8,   13306013701237763_i8,  853845923628244227_i8, &
  129343269068548019_i8,  387260726082950270_i8,  480699932129906808_i8,  168093623794049736_i8, &
  493964078889003228_i8,   91810820904715904_i8, 1063030002180550223_i8,  288206386754148555_i8, &
  399940785366423222_i8,  769238022069908990_i8,  922807617703668522_i8,  297818022576234296_i8, &
  412622773079980818_i8,  353728131878581430_i8,  943598845884248554_i8, 1024985783258294093_i8, &
  962436829150308343_i8,  911840453717962580_i8, 1069884958790991958_i8,  777896270352438579_i8, &
  736937238415801963_i8,  764342041144767834_i8,  605660074942814412_i8,  761836461189083962_i8, &
  444249247537243641_i8,  427639524689089303_i8,  936177972912858927_i8,  506667814354965122_i8, &
  928750604686779590_i8,  491495493339446384_i8, 1118823484261789249_i8,   18560804323826245_i8, &
  975150365234123243_i8,  962006759963872280_i8, 1099178705066228600_i8,  235878958679064462_i8, &
  622375685383196178_i8,  814061014909315830_i8,  454812756416206187_i8,  160262629597195795_i8, &
  802779153118968098_i8,  602121296111444185_i8,  711131189388092151_i8,  183804429027859321_i8, &
  359554033433896908_i8, 1107609825559640183_i8,  957290638892410867_i8,  281705739472745497_i8, &
  661747069931712171_i8, 1103258572185218502_i8,   45576809841499105_i8,  346133848303364021_i8, &
  364911859707881317_i8,  955816234341912338_i8,  652397200042599349_i8,  256979122796810422_i8, &
  467142890219077020_i8,  723141035467822570_i8,  268829697630100397_i8,  164303628632810429_i8, &
  498552902490770449_i8,  402957827878518594_i8, 1022648516208338886_i8,  478142083905146055_i8, &
  589673705573073463_i8,  826732819264445374_i8,  453732591445109520_i8,  108399403435545179_i8, &
  997796776742831170_i8,  100645588407184784_i8,  931241086950189668_i8,  789947348731725307_i8, &
 1062824809100264446_i8,  710381292173097992_i8,  459499635835143008_i8,  350471359457878913_i8, &
 1130534647625996086_i8, 1138504932131028737_i8, 1057063540428474172_i8,    1660225950415851_i8, &
 1002242388162658969_i8,  861738828415995720_i8, 1149224936363291887_i8,  254585989755123311_i8, &
  703626640314643573_i8, 1091371323874842542_i8,  803436833661748356_i8,  477636205185949220_i8, &
  463406278634657648_i8,  634256963748769252_i8,  916776836296049157_i8,  937059360910563146_i8, &
  693193149181780137_i8,  990957269172231235_i8,  163875509483012617_i8, 1084216066669412945_i8, &
  967990315052514589_i8,   52453636623068960_i8, 1092238295709824986_i8,  591505990259187244_i8/
data (mppicon(i), i =   400,   599) / &
 1052522681944133649_i8,  423367435060488704_i8,  195277722846504858_i8, 1119358325704123420_i8, &
  243277970169923253_i8, 1081276925563852910_i8,     105933928717437_i8,  441667678763103118_i8, &
  748534059479909058_i8,  100006890760923403_i8,  916164679351247310_i8,  401338678932029588_i8, &
  827538116120283709_i8,  561023343976658725_i8, 1113037397724676443_i8,  814370791516722501_i8, &
  394074848599857213_i8,  562163642904887781_i8,   67376635378596002_i8,  599345535363658694_i8, &
 1141037877017267908_i8,  243946432642182275_i8,  565089818679337858_i8,   82877817785943552_i8, &
   63114414730499066_i8, 1107035294448973228_i8,  372984429052141135_i8,  735058901755655177_i8, &
  701606885883743901_i8,  860459320219980231_i8, 1064820680785628187_i8,  482476804831651563_i8, &
  125654621631043400_i8,  770955328130358378_i8,  249618161836673107_i8,   71933230724278237_i8, &
  392668713470414278_i8,  185663248834208869_i8,   48577690340318947_i8,   20863977760662229_i8, &
  115514620963363462_i8, 1072468076759057458_i8,  301449414113995420_i8, 1093541589881854889_i8, &
  858334085349971280_i8,  839435060255025711_i8,  695588692014377365_i8,  607692750239593214_i8, &
 1140527066372521732_i8,  501754982910236686_i8,  335823319742153115_i8,   44590503809669438_i8, &
  689329308630833047_i8,  202393142171973813_i8,  114513776228793766_i8,  566640962104182738_i8, &
  948042489956118002_i8,  831395362660361640_i8,  645506647253086688_i8,  115508526105415646_i8, &
  978401091602597005_i8,  181187957687670370_i8,  682597907491741441_i8,  657199360099973472_i8, &
 1028197111911159045_i8,  494631314286944314_i8,   99401371877424579_i8, 1061126937193577001_i8, &
  515410286110967594_i8,  122713029722124831_i8,  429421305597975795_i8,  114651839437935967_i8, &
 1005713760570215098_i8,  273508487079730208_i8,  781929427076474828_i8,  781697743960765715_i8, &
  590214054562060386_i8,  662609245495601420_i8,  214834108693553614_i8,  452458912770853742_i8, &
  251853211068418270_i8,  375652825896061482_i8, 1011076659558884457_i8,  599787787263284843_i8, &
  259596938973135069_i8,  398232034620869770_i8, 1034630511187417229_i8,  291833613596932942_i8, &
  269052292399774297_i8, 1149636040661945291_i8,  346167435919796474_i8,  452695019793492715_i8, &
 1097874763438351459_i8,  379135634950596465_i8,  861962567683411469_i8,  639077601100400417_i8, &
  464345009716281408_i8,  914376870519948109_i8,  163934989336937697_i8,  792706692096958928_i8, &
  484312558848614472_i8,  501011955760368312_i8,  144425376169816615_i8, 1118600618699553075_i8, &
 1142994265260635205_i8,  169969141059294492_i8, 1042947454800607136_i8,  142897607827294187_i8, &
  920935506785669571_i8,  729172662926510196_i8,  708278717168802526_i8,  917978605756101532_i8, &
  228213058120945145_i8,  222873253531942777_i8, 1034059091971619570_i8,  959724059444402635_i8, &
 1123404204793592874_i8,   70122209191477586_i8,  430909135531194012_i8, 1061770368333720490_i8, &
  635489962604552965_i8,  477593822701182128_i8,  262408212101074454_i8,  688403907114306531_i8, &
  178148803656783154_i8, 1012528360912978754_i8,  831320890927680116_i8,  294064605113375719_i8, &
   81279705719299893_i8,  718563531623165536_i8,  823955390406401509_i8,  346114448956426524_i8, &
 1036524797657251439_i8,   99513595260486096_i8,  397600802542648098_i8,  693363299071976231_i8, &
  442359539068655308_i8,  923284992293217703_i8,  426634074140350344_i8,  947793101666527528_i8, &
  124211623711772699_i8,  512395926215355515_i8,  950956697818882512_i8,   30468737336122021_i8, &
  287494425167313701_i8,  864167055961581329_i8,  173676322760041142_i8,  917880883236160181_i8, &
  221026851445859283_i8,  255222856736340703_i8,   52416058391742158_i8,  533530295502243670_i8, &
   27168708955638504_i8,  638815889316840655_i8,  696507739442142210_i8, 1132840849344212557_i8, &
  498740645747554409_i8,  749231317999194706_i8,  991364654590021217_i8,  228812210073598205_i8, &
 1144823341402994746_i8,  914268108348696300_i8,  503096937643564364_i8,  861855622251829283_i8, &
 1077741448626252528_i8, 1040767569161345410_i8,    8807764107699497_i8,  840095131873034120_i8, &
  456769090322611276_i8, 1072243966380053006_i8,  322151313524637155_i8,  773987976946509093_i8, &
  987056743911026298_i8,  493643932638416845_i8,  660125598750882950_i8, 1119860095173264420_i8, &
 1075175910445779404_i8,  513354668034299888_i8,  442122892089012611_i8,  347361177477359662_i8, &
  264247774697419514_i8,  201174223151248813_i8,  140186384052173025_i8,  357450420738028801_i8, &
  335066712757483082_i8,  305764990370978606_i8,  292867629567733648_i8,  711791125801957234_i8, &
  869863439542552344_i8,  886783735113342972_i8, 1045841156130879185_i8,  196835175534531835_i8, &
   51532982163929509_i8,  259390229499896424_i8,  335551754210047672_i8,  298903780632381541_i8, &
  368124299874320777_i8,  959624407079722323_i8, 1029145421298505455_i8,  318071778594678750_i8/
data (mppicon(i), i =   600,   799) / &
 1071297798809568407_i8,  518992790295221002_i8,  616936618661776078_i8, 1060760059633173168_i8, &
 1017521720810691726_i8, 1043573831805115145_i8,  489181487301733752_i8,  641732161361706498_i8, &
  790033570088026856_i8,  971367919752476560_i8, 1050511046983989958_i8, 1151367868361306003_i8, &
  151919057777764520_i8, 1067952769870011321_i8,  710065525415532004_i8,  341936976146628828_i8, &
  362825178883977810_i8,  275596201093612068_i8, 1060138530090755105_i8,   43966104131745934_i8, &
  981642510550314903_i8,  539118837775256522_i8,  232758249934238083_i8,  591161896580232746_i8, &
  975875067442073077_i8,  934516619388227943_i8, 1082381477451456600_i8,  622951457146967118_i8, &
 1106503024727354785_i8,  567400702122704059_i8, 1046932860717838812_i8,  938257555016460148_i8, &
 1097285986120910461_i8,  262249223347657593_i8,  657132871808198372_i8,  372830029715726824_i8, &
  325168415081831683_i8,  104475676958710564_i8,  569149818378726201_i8, 1087772481220225532_i8, &
  992416400572751862_i8,  690444477756277008_i8,  100613230703668065_i8,  953931084086685434_i8, &
  625564345307896989_i8,  813867927041913424_i8,  902192666140827540_i8,  647720439546040181_i8, &
  136448434079390154_i8,  723734538083509480_i8,  337840266306612852_i8,  722825919187517635_i8, &
  504362442033402712_i8,  837219949044583649_i8,  914893802568537045_i8,  801587043494028111_i8, &
  394388132612581804_i8,  386930410710233184_i8,  296399048304558789_i8,  260133673861225909_i8, &
  850066080844741999_i8,  122248522314685408_i8, 1035994194036708682_i8,  467060437931421398_i8, &
  438423778608438674_i8,  660017893582929523_i8,  892747664873971525_i8,  551713310068024908_i8, &
  950428783514874250_i8,  632543407514153832_i8, 1111397300078008665_i8,  174139869594430607_i8, &
  935482183007308732_i8,  140867884308043868_i8,  565899223729731533_i8,  468283228320275276_i8, &
  581009777062092197_i8,  124727587478005720_i8,     957693647855020_i8,  727931981101339393_i8, &
  744353006463908431_i8,  827803025595318430_i8,  393099827202063966_i8,  573365264589170861_i8, &
  370467711105595481_i8,  706400161932801028_i8,  523306761183789525_i8,  140496182440362109_i8, &
  602654892638241360_i8,  854273130926188695_i8, 1071894502734959163_i8,  727605128467842919_i8, &
  586597537542562517_i8,  641634222948277344_i8,   40745008904822933_i8,  115812574431699933_i8, &
  932226805940754753_i8,  674240375350886297_i8,  247714204353199290_i8, 1071456624787147855_i8, &
  645210083095010568_i8,   66320281592083992_i8,  600648595949770448_i8,  955671375291449992_i8, &
  410423156555247693_i8,  319602796669768711_i8,  812675259004306178_i8,  475909500491494344_i8, &
  905517779594044918_i8,  898940397308968190_i8,  495034836871257826_i8,  727123010021176901_i8, &
  767113063022743327_i8,  904591403831536975_i8,   91786549998231265_i8,  132216142609666622_i8, &
 1094249398817683068_i8,  308515899861853897_i8, 1142418996837768150_i8, 1109644164580479236_i8, &
  851699040903447341_i8,  938271765022509176_i8,   10598230849177549_i8,  775201908538417177_i8, &
  895057427772715819_i8,   48278546484144369_i8, 1049267729932011133_i8, 1123400876065663929_i8, &
  860256900888456639_i8,  992871634825421964_i8,  929272593477481867_i8,  896427123266261437_i8, &
   36427989990237761_i8,  416237530499580845_i8,  461761675978468651_i8,  840399478064667520_i8, &
  395769322899414131_i8,  802398258443108679_i8,  518563240307624497_i8, 1088518482954630019_i8, &
  738151565092561879_i8,  150034689867971597_i8,  931199397573581479_i8,  538704065636387145_i8, &
   71187567004261412_i8,  445264343163873388_i8,  652351489650769333_i8, 1092773657793363046_i8, &
 1003735675027378821_i8,   94048941308685914_i8,  435943727121749832_i8,  596873906601242850_i8, &
  408388480839662965_i8,  911355713430166159_i8,  606431818365418355_i8,  874904933863273650_i8, &
  323676684129919256_i8,   64150339000351190_i8,  182592830486849383_i8, 1032652832708465984_i8, &
  582782088904130992_i8,  967586551128196963_i8,  484538236543982368_i8,  463865476535547039_i8, &
 1082122655787315500_i8,   64824798333089183_i8,   83242840952983173_i8,  830986217466038223_i8, &
  483193039926455592_i8,  665944119556224225_i8,   84850718232333231_i8,  605303142101472638_i8, &
  187267547820190628_i8,    7508796889304933_i8,  400569341598891134_i8,  691748288114339749_i8, &
  504989974784528163_i8,   38508400151925637_i8,  470369294719372532_i8,  116232796450372843_i8, &
  849536437153813566_i8,   77983885405741242_i8,   36631071043607096_i8,  898707438408351751_i8, &
 1063037910526942402_i8,  611661644155963484_i8,   15352133684695822_i8,  769056555843724319_i8, &
  792693057289603771_i8,  397360794136385185_i8,  779242029728093169_i8,  310139790379785256_i8, &
  180860703526638733_i8,   21749322919107328_i8,  519302453209073774_i8, 1138895943637119413_i8, &
   38711698915228100_i8,  719993131856928946_i8,  790578719940331802_i8, 1083100657525698823_i8/
data (mppicon(i), i =   800,   999) / &
  435921859080409804_i8,  683859700350302683_i8,  224408844934079738_i8,   64062858055166832_i8, &
  557906537221420365_i8,  515881141598943277_i8,  413750453503730423_i8,   51322907665264401_i8, &
  964696595234198504_i8,  232592790637985813_i8, 1102877689642802589_i8,  137464178751582378_i8, &
  485837969167564451_i8,    7061694381170294_i8,  656613522942125872_i8,   71405825786382508_i8, &
  228111626328775022_i8,  370342730481644567_i8, 1118464641136427772_i8,  809396842274320332_i8, &
  968747591202856048_i8,  545572763726792532_i8,  114022411656886261_i8,  750715224150815279_i8, &
   75361024281878212_i8, 1103183575411308858_i8,  317494798189727881_i8,  726211830069553748_i8, &
  848392752301587238_i8,  515837948444824953_i8,  500067048703696521_i8, 1137986868193745443_i8, &
  600978865535978095_i8,  561631987733100716_i8,  304346817339418554_i8,  551744473249252034_i8, &
  696672977504067279_i8,  862252903688135965_i8,  181077465951216997_i8,   13816519969146727_i8, &
  596870961726389703_i8,  370337548752297798_i8,  101271545835159262_i8,   76090136150498517_i8, &
 1013086142418967041_i8,  300273600868137964_i8,  655016978566379688_i8,  195839269992104576_i8, &
  674983923591804060_i8,  804994011941200673_i8,  325890055801692942_i8,  165846561741549189_i8, &
  942921890419373924_i8, 1002919343762028755_i8,  146813462418537765_i8,  754533952790608511_i8, &
  681188917026544196_i8,  861851659122373904_i8,  337173256059144399_i8,   20338778714613003_i8, &
 1035579202838301492_i8, 1104526176869871042_i8,  560900020457962407_i8,  859766629653011101_i8, &
  546149027385075786_i8,   55941649597302435_i8,  284019497721383241_i8,   53128470725581510_i8, &
  306213346829030544_i8,  737850965785264078_i8,  501376169270697070_i8, 1070932720730191216_i8, &
  156427127977381302_i8,  878545584687233630_i8,  714423476050270602_i8,  857520761046882670_i8, &
  151876277936961589_i8, 1152057274154960867_i8,  813360104868515020_i8,  246076687664095957_i8, &
  756271332343014605_i8,  666741679751712501_i8,  916496972024055849_i8,  887164017613808388_i8, &
 1022544735925390345_i8,  866030006550386156_i8,  433437555791625681_i8,  392115846733673325_i8, &
  838167602632271011_i8,  265963724909522856_i8,  518367335995547107_i8,  862341900825383830_i8, &
    3822883952496752_i8,  899504826901541836_i8,   99124922061942241_i8,   99286404248652475_i8, &
  963951179419665801_i8,  722321218269613703_i8,  582396237363530859_i8,  713207056714163153_i8, &
   87572713904423990_i8,  751530305399325468_i8,  469830716649184948_i8,  258635271568531384_i8, &
  717193941637982838_i8,  818257497761904645_i8,  913446176381770860_i8, 1110387277686847617_i8, &
  355073235246730091_i8,  987168387766052359_i8,  199365090686538939_i8,  461606783862269064_i8, &
  395640949531528740_i8,  611886440921089174_i8,  612199002052626802_i8, 1100888007597249181_i8, &
  184967050614576171_i8, 1081108654420748584_i8,  934217096057883641_i8,   72527755808771929_i8, &
  197793939771527617_i8,  836825914087774701_i8,  588568863675067147_i8,  977227188649590203_i8, &
  652842651161834092_i8,  108439542847846691_i8, 1049916010542913886_i8, 1072932622883929081_i8, &
  545457576712216839_i8,  910148148247261345_i8,  412171042383460579_i8,  643963586267682386_i8, &
  693349082513547396_i8, 1025723163039371207_i8, 1099869253417316512_i8,  193825348936926842_i8, &
  991672070874240695_i8,  832104773549101279_i8,  299036973101102924_i8,  478730593115693954_i8, &
  528855334168795017_i8,  207826200297386623_i8,  169233128223864120_i8,  384995516814316025_i8, &
  240480589962934588_i8, 1041274463433539864_i8,  327055291618690086_i8, 1052260099830717042_i8, &
  961846811081948106_i8, 1098401844393556361_i8,  615353155778303215_i8,  512633924258508306_i8, &
  819061000620201966_i8,   97278518409877913_i8,  574259243111728771_i8,  914708573921563556_i8, &
 1122196832129288617_i8,  600624207280723695_i8,  234708564322242556_i8,  331885530727851390_i8, &
  568138828831247153_i8,  515803018093970379_i8,  469977572478525816_i8,  638255583681584054_i8, &
  270209934941068610_i8,  731280974035176986_i8,  878106903318701483_i8,  535468971829388992_i8, &
  701959402678572657_i8, 1086308637726031787_i8,  914294756369509759_i8,  475183880396342175_i8, &
 1086510848495192633_i8,  353753591940524302_i8,  917037902115425609_i8, 1043414896378705581_i8, &
  855226531306980668_i8,  632579447240364063_i8,  907700399667344235_i8,  971185566741072906_i8, &
  874750547894182625_i8,  575483041571081879_i8,  909402777220934393_i8, 1083501709858291387_i8, &
 1096585993983468443_i8,  139704801708340731_i8,  284001923813218699_i8,  171473015951168066_i8, &
  185596178009345321_i8,  861902696686557701_i8, 1080837709014526528_i8, 1077634080909673371_i8, &
  653437640175444861_i8,  228807947704407030_i8,  592098790577148796_i8,  890067626935423168_i8, &
  441537495795765413_i8, 1060671578553127197_i8, 1074207413264856389_i8,  434639518912097367_i8/
data (mppicon(i), i =  1000,  1116) / &
   91502086598018387_i8, 1055686388652374320_i8,  557190800486253063_i8,  815439364490083386_i8, &
   84966373018511719_i8,  276995072069481162_i8,  220526751575517413_i8,  972260112228629558_i8, &
 1085521452294961529_i8,  713027274959290285_i8, 1151516579614649520_i8,  660366430030084098_i8, &
  460660278415423450_i8,  454290926768519759_i8,  944170765619993852_i8, 1093819913056384742_i8, &
  242311117027896833_i8,  569777999001083134_i8,  281300663479999602_i8,  656176045708108361_i8, &
  864399413967615071_i8,  781863448895587594_i8,  222424586086972759_i8,  851412490298422381_i8, &
 1127281275731160733_i8,  406541390120931870_i8,  752259664563816305_i8,   72705718391831624_i8, &
  236686916035121382_i8, 1111809480857157650_i8,  602199136978509852_i8,  684847631953943878_i8, &
 1141132800083243126_i8, 1126979914804451964_i8,  560176389796043205_i8,  113167092493977553_i8, &
  907862631694763588_i8, 1078044950378582581_i8, 1106013951213258845_i8,  846418050553168074_i8, &
  192386549681314049_i8,  844950000941166953_i8,  247728967959939618_i8,  458636325193080767_i8, &
  671542013742435065_i8,  102993857964978993_i8,   39150035991658758_i8, 1152660213212165215_i8, &
  290334734983060691_i8,  735904953723145947_i8,  104032781304093246_i8,  249831666172095710_i8, &
    3154090832205159_i8, 1109732315068353250_i8,  144644884193264485_i8,   70214458438594779_i8, &
  832326974180709953_i8, 1070981139929216902_i8,  792475385880950006_i8,  155375900487983189_i8, &
  154167715879877523_i8, 1110642083395673325_i8,  599777367207642692_i8,  563246372342768852_i8, &
 1021900770567761848_i8, 1141508483825033964_i8,  915100124885560396_i8,  961242267149702421_i8, &
  750248093756832303_i8,  895257227468808830_i8, 1074771164705836571_i8,  121619851849489802_i8, &
 1142214762738312275_i8,  256492664737530673_i8,  617375037921663163_i8,  509350389270748757_i8, &
 1135804246176955933_i8,  777497502310497120_i8,  630172658511656645_i8,  378915530885345103_i8, &
  377730987277820196_i8,  512732864286521778_i8,  384560264588369116_i8,  674904731320476002_i8, &
  870295562703256141_i8,  603332192725907287_i8,  922245186974062365_i8,  594450130570465346_i8, &
  632093663824517706_i8, 1093188729195098730_i8, 1010470117276801206_i8, 1062685030377708486_i8, &
  732010324935348387_i8, 1147732280164422973_i8,  616174040908811045_i8,  421595017181802079_i8, &
   65752240039950205_i8,  131322276630060995_i8, 1100059463454792783_i8,  174570284100925599_i8, &
   50680903182113732_i8,  848996591539105372_i8,  830800796531764614_i8,  368387844562805881_i8, &
  557769924498866829_i8,  589651025514406575_i8,  817121523055458575_i8,  767084900178280219_i8, &
  616038697526650331_i8,  458126119918118820_i8,  536288130006954897_i8,  952969870740457625_i8, &
  466921250634065894_i8,  637809219400435158_i8,   39839887081268338_i8,                   0_i8, &
                   0_i8/
data (mpegammacon(i), i =     0,   199) / &
     140702047753744_i8,                1111_i8,                1111_i8,                  -1_i8, &
  665484352860916858_i8,  347267826240803531_i8,  390156120941880780_i8,    7050085186957466_i8, &
 1151752606534574138_i8,  716334919560514402_i8,   73602480980289811_i8,  149562206868686451_i8, &
  969605422653500295_i8,  634029906398537663_i8,  252576183713573229_i8,   96397043423906825_i8, &
  958294114473278762_i8,  137955613072712766_i8,  899637010264191525_i8, 1038857003287562054_i8, &
  675913959469203109_i8,  987783547075159760_i8,  718601390998323286_i8,  550702484374349633_i8, &
 1035130444846633756_i8, 1069642309481089380_i8,  829674530617867468_i8,  698930240445215274_i8, &
  954452585256823596_i8,  523700867325342427_i8,  332955408228578334_i8,  376162650432507504_i8, &
  339957742694499524_i8,  976242166247851132_i8,  384266313615857870_i8,   80486734971496130_i8, &
  890907389908540263_i8,  697821067421430265_i8,  867763285472509589_i8,  382494982594998366_i8, &
  410485221065951521_i8,  428811319869492329_i8,  999543124565046305_i8,  575545973646342462_i8, &
 1104377048525858334_i8,  415015423285684411_i8,  914206752641652925_i8,  218242964768942540_i8, &
  275128044539179450_i8, 1118260183855495297_i8,  872089622909092777_i8,  197967216122738918_i8, &
  379122566301540698_i8,  252682569276473974_i8,  231967038614344371_i8,   22829378533120003_i8, &
  441715245888220069_i8, 1097285772704393854_i8,  872410864413589326_i8,  140632378443246352_i8, &
 1009969065122592190_i8,  564159868118165404_i8, 1109967291986150486_i8, 1123705295439205987_i8, &
  220604870331055657_i8,  826737927311389737_i8,  633123876397497094_i8,  770797308803459558_i8, &
  235325246329041491_i8,  440003421009095187_i8, 1026740077502789243_i8,  671427397485161127_i8, &
  803114939033395436_i8,    1836620429020796_i8,  994798803901402676_i8,  734946187693224066_i8, &
 1055853531906229654_i8,  429919911900196640_i8,  574633893417037271_i8,  821603132926683200_i8, &
  987610570127154444_i8,  751311884506913944_i8,   14466471017442102_i8,  831354056492040531_i8, &
  580061672397717002_i8,  437922793675275285_i8,  586891388917503897_i8,  868655563755155117_i8, &
  198618710191170246_i8,  358688896132313469_i8,  316753307027363914_i8,   17094053218720000_i8, &
  758669241089701968_i8,  158378840570050485_i8,  898210974090411309_i8,   26106385996947381_i8, &
   85942012097505010_i8,  819516267225268035_i8,  584141088014358687_i8,  345187771504731719_i8, &
  651297745842137871_i8,  918218514473303806_i8,  331431211151682476_i8,  653667942657868606_i8, &
  103278526254212345_i8,   48244851128203670_i8,  511987854797925091_i8,  444508543932083845_i8, &
  332326529281534069_i8,  601049181819071046_i8,  754518066069823798_i8,  705473184490739575_i8, &
  373565873634988874_i8,  510763700237885090_i8,  677240631479080343_i8,  309302015440896812_i8, &
  298081365784831500_i8,  108753016564078029_i8,  775598311723686238_i8,  589044855433810862_i8, &
  256169169997696023_i8,  345224161009423726_i8,  456337241973199508_i8,  201002137203716344_i8, &
  775095023135802155_i8,  832624222988329863_i8,  205198957884103929_i8,  796260718343911886_i8, &
  737904148786352325_i8,   22163881326324884_i8,  114903724820203580_i8,  251882491776402488_i8, &
  152430832197246232_i8,   65059886775496781_i8,  369274324726301248_i8,  211176128295051633_i8, &
  387202142434056825_i8,  567069143155548455_i8,  654697219848651030_i8, 1013707855470886710_i8, &
  538456231759695070_i8,  542819555642847629_i8,  841155595610822032_i8,  191980338865396475_i8, &
  923637565691441542_i8,  663081043070146769_i8,  575910027365356263_i8,  845559026468905263_i8, &
  965717485382031441_i8,   47331232843644030_i8,  276796454520794818_i8,  353660737325890942_i8, &
  592657130123009067_i8,  376772355886827456_i8,  676795046804908331_i8,  396996508482024292_i8, &
 1015799601637566018_i8,  842550551795555939_i8,  763424224638379449_i8,  392736376257861238_i8, &
  542026903097859742_i8,  387560259063140364_i8,  246889305716000609_i8,  511570559803604815_i8, &
 1024423041876796169_i8,  416860794721304009_i8,  741471053427597922_i8,  171447623340430759_i8, &
  389986454713968917_i8,  770925970177926483_i8,   50568999576542222_i8, 1069983168537339771_i8, &
  343652398945532970_i8,  889759809637076025_i8,  137899803722969832_i8,  409559720496625343_i8, &
  286613983630280950_i8, 1122128759995852225_i8, 1109217513122736502_i8,  885049477663161056_i8, &
  756291993154638955_i8, 1011098035082839769_i8,  775931341328403876_i8,  443594335863896862_i8, &
  189257966454843882_i8,  438807794424936682_i8,  919476125141280586_i8,  888054514746675646_i8, &
 1017601970342846428_i8,  537352454583294730_i8,  759097692760811565_i8,  854529768681310867_i8, &
   13852463175951891_i8,   17468984991535141_i8, 1009386978064274111_i8, 1115457362323040414_i8, &
 1135556027671751379_i8,  151229895662161657_i8,  602939512173916171_i8,  137947822996997666_i8/
data (mpegammacon(i), i =   200,   399) / &
  472198810277513340_i8,  683459015045856861_i8,  788575094071880139_i8,  981485422661197130_i8, &
  848043825049369191_i8,  524289617526281350_i8,  151400659737842599_i8,  658680609793858822_i8, &
  297827654594411538_i8,  834068296006724221_i8,  203281972288431766_i8, 1077902215791588639_i8, &
  102704040999402028_i8,  862259605194699973_i8, 1051995326569477900_i8,  728652192748889708_i8, &
  420126655934406859_i8,   19889884056872522_i8,  276051156542793677_i8, 1034400850969825147_i8, &
  340051675075176187_i8,  766474278622922387_i8, 1083248606994761801_i8, 1059303261966213847_i8, &
  177175131508542254_i8,  660868795343918828_i8, 1029236240265290477_i8,  816059003305835115_i8, &
  741820224566234238_i8,  673860369066328750_i8,  582543568378823663_i8,  261750765974145930_i8, &
  442931809577254497_i8,  225939131966871691_i8,  412115679324563466_i8,  129746436378425616_i8, &
  746340371297275021_i8,  846108284077969005_i8,  733932099673835130_i8,  656223616547915376_i8, &
  550395129023657835_i8,  740366921726258227_i8,  521022032371149632_i8,  282339127424105056_i8, &
  764307237785101086_i8,  191533832915484765_i8,  111989387903547695_i8,  153304422613750110_i8, &
  420252932020364157_i8,   28347326109901532_i8,  501091903122366813_i8,  299667363213729028_i8, &
  307617136903795031_i8, 1103762814936663112_i8,   79356011989172108_i8,  149628486504525004_i8, &
  530708636163202995_i8,  930569966849951530_i8,  344691712187363875_i8,  296510379771732563_i8, &
  160138730674440060_i8,  308731749554836973_i8,  615527803655613212_i8,  924680935705507088_i8, &
  775025058576975152_i8,   72637327853661241_i8,  304015062660387101_i8,  152335373999633115_i8, &
  976492218209201463_i8,  520381140697160819_i8, 1086816552594398302_i8,  299988288777219221_i8, &
  885533292067710079_i8,  843264164092642639_i8,  752606198917404655_i8,  643722710022427206_i8, &
  419794749121367032_i8,  276809227315667712_i8,  804984249884026311_i8,  831199518279278473_i8, &
  619114346413557650_i8,  409481642831109619_i8,  493094919924389677_i8,  625366396586621457_i8, &
 1115902926143869528_i8,  379661208849537857_i8,  110588952918802155_i8, 1024389007548859851_i8, &
  720892279345233918_i8,  624898104810107438_i8,  190459485267289104_i8,  496538013290776596_i8, &
  546728285043287719_i8, 1100121912763828301_i8,  383381393861736162_i8,  346008451001390588_i8, &
 1040630124893114205_i8,  157497185297950522_i8, 1133085510072031865_i8,  340552992883285605_i8, &
  161924037689786519_i8,  722393035448998151_i8,  813017467435163868_i8,   55460526849491686_i8, &
  348601473260420035_i8,  524961685701894168_i8,  452685712385850831_i8,  949670614815056552_i8, &
  445963938461491929_i8,  797989089634200000_i8, 1028885457774774757_i8,  861516323082716813_i8, &
  813592412700196439_i8,  649899242825104835_i8,  845717749854945745_i8, 1043075978262982931_i8, &
  773322747828607479_i8,  790704973510581718_i8, 1127914558303272108_i8,   31163552216137645_i8, &
  564512620064481155_i8,  366882517681955886_i8,  675069568247427581_i8,  108510628432620539_i8, &
  798265523576380659_i8,  124586442241891443_i8,  584506519698754780_i8,  171986932189927048_i8, &
 1034670724400113936_i8,  646200773188252545_i8,  284304149500716779_i8,  839292891749784928_i8, &
  223928951870294162_i8, 1035326250369328182_i8, 1050807586229436876_i8,  127861317699815604_i8, &
  145158407530037903_i8,  390133730819177979_i8,   76800468496479279_i8,  469177789002729116_i8, &
  575693167089298873_i8, 1127323716763135597_i8,  519286115434999265_i8,  274490574238619618_i8, &
  119288038749008721_i8,  368730540633912477_i8,  655491894794488718_i8,  624034038925633391_i8, &
  925598606882322052_i8,  414468905534141320_i8,  575554016931333813_i8,  585134707105796939_i8, &
  602107224094761896_i8,   24746472711503702_i8,  533289544483989695_i8,  963120955603637584_i8, &
 1069840974686913186_i8,  861987817214147261_i8,  157470341995769589_i8,  631597550453667589_i8, &
  275327368693174827_i8,  731145016360520200_i8,  721389940030098894_i8,  798593068233079424_i8, &
 1066458116214923695_i8,  458197060461850802_i8,  110540160940360122_i8,  938005304251830941_i8, &
  768134551176008032_i8,   61544834167012439_i8,  770098145967226031_i8,  333400315616518721_i8, &
   71052255987938549_i8,  916150249626144662_i8,  712377242238404313_i8,   78587376899392445_i8, &
  270080246153280012_i8,  467537165312513812_i8,  432661599617319182_i8,  627952010597926762_i8, &
  146589353001965149_i8,  242223052963464539_i8,  949694822983236320_i8,  576613447882688132_i8, &
  647102211405837400_i8,  771396840104402100_i8,  554372884190383793_i8,  433422853690538983_i8, &
 1118567937678818084_i8,  659985562215412360_i8,  454353026957942483_i8,  844263090744689063_i8, &
  404588361454685821_i8,  548135446248769449_i8,  463573834438496076_i8,  771885023275555118_i8, &
  177279017828188725_i8,  569113577157967321_i8,  272779414250936734_i8,  951183470965359554_i8/
data (mpegammacon(i), i =   400,   599) / &
  136151248168236171_i8,  446384002832737047_i8,  506361599511244502_i8,  713452921393109847_i8, &
  839427357355157929_i8,  777716641493228535_i8,  467016485996647071_i8,   90737250865760395_i8, &
  670309896754976370_i8,  893782018347459059_i8,  737491898016029376_i8,  779135267088265469_i8, &
  866250084164770364_i8,  293464509388825822_i8, 1070720878114994105_i8,  435989222853085269_i8, &
 1151988899539215588_i8,  112224329451437051_i8, 1053822999051129772_i8,  229668811128878339_i8, &
  894629955990184652_i8,  717837141979966059_i8,  822622582338306174_i8,   30427576178021725_i8, &
  378066700856844670_i8,  455228842937763285_i8,  242030552042465974_i8,  591505006911186982_i8, &
   67333208905009271_i8,  659570477017813745_i8,  699622931479408728_i8,  700585334677142675_i8, &
  569105871131015446_i8,  116319792732845898_i8,  354524749133485768_i8, 1064894131630408859_i8, &
  929540316626700589_i8,  748202109266805558_i8,  557624664240481357_i8,  433533383682283216_i8, &
  147348752373446100_i8,  252726055113341004_i8,  985127267639782287_i8,   74007444634201978_i8, &
   87690354098869064_i8,  293468610473914797_i8,  994278635269932694_i8,  934559466983735986_i8, &
  758639019980693943_i8,  360917364744194548_i8, 1072591692673686590_i8,  828684912661069239_i8, &
  681701880804130174_i8,  957716374950314760_i8, 1131125458745933399_i8,  941453112267169481_i8, &
  679026682289608472_i8,  828086818572322380_i8,  542757544098537582_i8,  321505815754895477_i8, &
  170105027971710460_i8,  591757996553762503_i8,  900294863449050561_i8,  702155322723967532_i8, &
  789298196596271537_i8, 1000291758615204803_i8, 1115256146007630838_i8, 1030180772603960583_i8, &
  462165821029603565_i8, 1135077210526537561_i8,  518308205084778579_i8,  126013584883680170_i8, &
  729423331499471497_i8,  808396104371551697_i8,  170555251386704475_i8,  448027932332735929_i8, &
  561969350405339389_i8,  400830466521947693_i8,  319820536751300531_i8,  861025158612784993_i8, &
  295858993110600777_i8,  791194391076106609_i8,  955217262298605665_i8,  160029173935973631_i8, &
  400575015288899587_i8,  180554867250581221_i8,   81626698431962388_i8,  261517212443945515_i8, &
  190873852942553045_i8, 1083061080088524879_i8,  244728197071631011_i8,  163786840439431973_i8, &
  152678550335107823_i8,  272846741320796813_i8,  204611375339476140_i8,  514860010916772339_i8, &
  673617651300155436_i8,  292467318959952293_i8,  140232971527135913_i8,  608293617367836515_i8, &
  830504011058271094_i8,  672740693158751273_i8, 1106704774900464548_i8,  802731547763795969_i8, &
  275649800342625566_i8,   81158175487645456_i8,  485972253617893058_i8, 1082166743376376113_i8, &
  485176676158015339_i8,  743268146486773419_i8,  396509331701263133_i8,  246591189442342687_i8, &
  260366483759734370_i8,  203166859051128457_i8,  910075979254130060_i8, 1077337669387839133_i8, &
  783328929813763005_i8,  588074378884226330_i8,  252600076967612451_i8,  939993571338123292_i8, &
  517892797260922050_i8,  394088613508079969_i8,  186561179775638621_i8,  586307791058325557_i8, &
  662799096245448855_i8,  131230889985951255_i8,  395807902561567720_i8,  737127827409574134_i8, &
  303329908248400098_i8,  186173193110147302_i8,  893653713590839947_i8,  455060246465846228_i8, &
 1093813232653261159_i8, 1028339678480535063_i8,  374691876551805830_i8, 1145168780065284290_i8, &
  198843471287750094_i8, 1115159120970478792_i8,  620909215502360175_i8,  715939567810377412_i8, &
  122568326154281599_i8,  386420022739266574_i8,  508489533947793847_i8,  249594221660476059_i8, &
  935231222957425385_i8,  815821248962399996_i8,  717869142489255044_i8,  487843864803103228_i8, &
  444930471105220418_i8,  929867909916285196_i8,  538185783186037673_i8,  467113038974838829_i8, &
  164336508778361106_i8,  688293381466366466_i8, 1031806037928342651_i8,  970444346379426097_i8, &
  692578256757931670_i8,  412398571184650420_i8,  239915731280189903_i8,  713555399008988737_i8, &
  550722509172554094_i8,  296339777122313604_i8,  382310785177914166_i8,  217970596813606565_i8, &
  561895282775591110_i8,  546474666251703420_i8, 1043009163653810581_i8,  821999773016962388_i8, &
  707589040281641926_i8,  838086366736913270_i8,   31649123951532030_i8,  726269973356564755_i8, &
  203459391948459818_i8,  984768223869704994_i8,  258736206958301481_i8,  923313024497778036_i8, &
  509415064177137125_i8,  964408876852339759_i8,  610642381622974747_i8,  635640670349836907_i8, &
 1144645043665077032_i8,  663029878818235582_i8,  322885011776367329_i8,  264862323645029469_i8, &
  792996900452456081_i8, 1013338330183487950_i8,  526658805555995601_i8,  701628868758225760_i8, &
  620065196981780993_i8, 1131239753124544941_i8,   65950960363304837_i8, 1126769703294651955_i8, &
  403546794294820225_i8,  303857723409438689_i8,  121987901621936709_i8, 1105962112979327601_i8, &
  772678285081349660_i8,  678011709802675643_i8,  701808911953812705_i8,   95799866787826536_i8/
data (mpegammacon(i), i =   600,   799) / &
 1019376401950328591_i8, 1024220605443144158_i8,  815724233705409648_i8,  626216755266610038_i8, &
   69175506275456392_i8,  160116328200060086_i8,  261047020563961590_i8,  798896050438457919_i8, &
  741440446326875928_i8,  536214085878376436_i8,  117365345985964845_i8,  983797856890309042_i8, &
  767950706516312975_i8,  254364369319845307_i8, 1039808796123396858_i8,  313365427282796249_i8, &
  269599957515995149_i8,  192830775671791487_i8,  844589504020202121_i8,  388580295526852786_i8, &
  557625179878665803_i8,  796776670474970781_i8,  587558011825866151_i8,  628614516825010181_i8, &
 1095600787955394480_i8,  551033500013114285_i8,  248044551186778679_i8, 1105931373912168660_i8, &
  204050408307159280_i8,  747588293554281921_i8, 1142435763679847534_i8,  129302568501197905_i8, &
 1085951330347581698_i8,  665429883108040510_i8,  692762807577570661_i8,  362861869574847253_i8, &
    3331362598667294_i8,  197316112587962650_i8,    1048075779360162_i8,  784293509231092373_i8, &
  195436901477642564_i8,  267310311582226140_i8,  983066764216998580_i8,  499388158231780360_i8, &
  700546638137103045_i8,  219678589985759452_i8,  976560423824880341_i8,  700784216395567333_i8, &
  442357990090710913_i8,   54286535324630957_i8,  408511825849659036_i8, 1147283156559035332_i8, &
  473583328397958812_i8,   89914259323369321_i8,  817884280084656960_i8,  931762181084420949_i8, &
  146755259144020782_i8,  986674214633291645_i8,  921533994447081864_i8,  530704797817144324_i8, &
  504327367162420958_i8,  978730082211859070_i8,  994706680664932039_i8,  625886575078614912_i8, &
  964780959697973986_i8,  578570271424505417_i8, 1054588793697066752_i8,  272545946884891361_i8, &
  682334150595376087_i8, 1065630581099403224_i8,  619947685182429680_i8, 1050201690753781623_i8, &
  345641201133070129_i8,  387348057806801456_i8,  386902083826451673_i8,  789660197464034526_i8, &
  651163817173773864_i8,  988625003359541059_i8,  375199389593802505_i8,  524085940822793789_i8, &
  382988290598611345_i8,   89827740270624516_i8,  370797026115096085_i8,  704531581304924956_i8, &
  485226397448427870_i8,  359778966824078990_i8,  846703178259629796_i8,  234266873363771694_i8, &
  446395142713272992_i8,  727293263220882015_i8,  368266328251450808_i8,  269559347754965103_i8, &
  205221983659225853_i8,  334954247661143828_i8,  706823870359142712_i8,  251818622329947296_i8, &
  558093594672339754_i8,  170862240732257347_i8,  133718861122064372_i8,  800664606387732525_i8, &
  919560994743936931_i8,  669616751654907738_i8,  921818142412244376_i8,  606218993531219182_i8, &
  118168665603462187_i8,  961895518323756563_i8,  551057483986314094_i8,  605403482499815974_i8, &
  541016979015450276_i8, 1101805875279880976_i8,  537800272675499901_i8, 1023462001080951872_i8, &
  261646676671890884_i8,  129057896190087861_i8,  903265574950757067_i8,  902194225757274691_i8, &
  850474530574238103_i8,  795640795499974472_i8, 1113042591623575235_i8,  576328145458893321_i8, &
  525996493023547748_i8,  881925415234096782_i8,   33666778231858184_i8,   75613264264412517_i8, &
 1124343700786809051_i8,  576601355562348871_i8,  604647080333206028_i8,  868014799082575985_i8, &
  377579721389301904_i8,  954139955411555614_i8,  778836876674379480_i8, 1134293483285446803_i8, &
   74755509967286371_i8,  552004769996039985_i8, 1150104421222947187_i8, 1042273258453578982_i8, &
 1100416454246614133_i8,  956877039909100669_i8,  723500937389482373_i8,  403437386866826086_i8, &
  340474871816577723_i8,  194159764596103925_i8,  799731426977064559_i8,  379483452996381737_i8, &
  884090901423903019_i8,  634776816224351186_i8, 1028672567979654914_i8,  364520762601546864_i8, &
  894476693414201988_i8,  497055046574857765_i8, 1082118181076647181_i8,  114611558653608374_i8, &
  809430924474411919_i8,  579170690056353129_i8,  254329060659415420_i8,   87157196663942476_i8, &
  548051508469210872_i8,  104926817003552761_i8,  645412500519798303_i8,  786610299235375418_i8, &
  569297719672768928_i8,  949194138469442752_i8,   77458840587001380_i8,  838746119144295326_i8, &
  958533983243797443_i8,  182821718143185360_i8,   47493154848492373_i8,  720640026270928931_i8, &
  939126634312388260_i8,  930904842486588046_i8,  798794373905596175_i8,  936901476515832285_i8, &
  527471721356444677_i8, 1052533022795129929_i8,  942694060497335703_i8,  598026023575689889_i8, &
  400779505464005290_i8,  647264747846589521_i8,  115575246612212602_i8,  808187207474409519_i8, &
 1099799745109503704_i8,  923253895799274876_i8,  941064400625179661_i8,  532768722595477960_i8, &
  911172190932059384_i8,  549282670876282500_i8,  699449658694722521_i8,  224418549773866444_i8, &
 1091369838128744229_i8,  290359259181241680_i8,  612652407048337687_i8,  103405140964470756_i8, &
  112959972974097651_i8,  441636535601419345_i8,  104253379575101978_i8,  266806635861255687_i8, &
   97596150976887464_i8, 1010862439633627706_i8,  159466406880759842_i8,  886621248594487567_i8/
data (mpegammacon(i), i =   800,   999) / &
  212375940631622930_i8,  218222717494128094_i8,  750851127949136626_i8,  223126204652004699_i8, &
 1024240959507534775_i8,  805192906770305624_i8,   48134130970069396_i8,  543634306504865264_i8, &
  956531836007572719_i8,  177358605251361897_i8,  225351980663003121_i8,  835849542271648543_i8, &
  985514560014956360_i8,  741738593465320590_i8,   95708021043144494_i8,  927509273486963100_i8, &
  545445248160239661_i8, 1144515781767393199_i8,  821329980130791796_i8,  114378216748609439_i8, &
 1026780565505106966_i8,  267425517037052633_i8,  112484968990427815_i8,  391333432242002164_i8, &
   98223696656069529_i8,  602514371627322777_i8,  524670324441460827_i8, 1038381634147779582_i8, &
  873242956653902139_i8,  315663350070263147_i8,  494946425534065219_i8, 1145623256593192036_i8, &
  194373640583707045_i8,  916306268950572915_i8,  270306093106347182_i8,  963682008429714520_i8, &
 1050155119871094755_i8,  648299748380780347_i8,  474238603232690260_i8,  797164699467944663_i8, &
 1008917079006721491_i8,  120406270783813582_i8,  184033285174525274_i8,  824953433554482158_i8, &
  875114762303765471_i8,    5492304541741530_i8,  707428028676097959_i8,  786538312553821185_i8, &
 1028362111646560228_i8,  761130844566264087_i8, 1111114401481321241_i8,  611973447630540415_i8, &
  859522179645650844_i8,  601724683516704627_i8,  287884967946585803_i8, 1023862970373937661_i8, &
  852927117505289396_i8,  328335919317252514_i8,   11571137033347944_i8, 1024604139026850369_i8, &
  747719518486290004_i8,  385630402413357464_i8,  363959610907476496_i8,  823307306093360227_i8, &
  657630180177489088_i8,  139549421253081381_i8, 1017245433534584941_i8,  512359693978971198_i8, &
  643029609276000585_i8,  359748307932854672_i8,  923608454817919702_i8,  193206062553554676_i8, &
  104457680965002242_i8,  714488009741316383_i8,  513669810061343300_i8,  354530941130381193_i8, &
  622443831331366322_i8, 1069532964695435174_i8,  856435901731852548_i8,  631267962916839863_i8, &
  574407797596935163_i8,  598897296542362947_i8,  885823437751380420_i8,  875449861117869715_i8, &
   44947619944029278_i8, 1085370531774468559_i8,  399268192498803656_i8,  771931708197393230_i8, &
  980686322785219913_i8,  440139751985963561_i8, 1057551401969654865_i8,  123178851482878560_i8, &
  732371299977701178_i8,  383140219046665236_i8,  592501454860603671_i8,   47245122747939831_i8, &
  602466200348604077_i8,  854090132497774861_i8, 1130561388078574566_i8,  808743149177709769_i8, &
 1122955839070029616_i8,  566855580828352482_i8,  787541499094791666_i8,  326696007116676647_i8, &
  728855799089576987_i8,  340978737600672462_i8,  644602411867377897_i8,   88673244035913982_i8, &
  934175662359984476_i8,   55617545164867238_i8,   69322887882775080_i8,  909585904011291434_i8, &
  284905996423501330_i8,  580801912942981359_i8, 1071427418749922419_i8,   14808252687044148_i8, &
 1062284770588543837_i8,  117767419816170762_i8,  268788849050418311_i8,  978963234448218813_i8, &
  433491652162588180_i8,  437013571490824061_i8,  348683607330537659_i8, 1003713563915606096_i8, &
  578429293923998197_i8,  312706831911697832_i8,  713060487654975219_i8, 1099891329783342450_i8, &
  541224996738416153_i8, 1082017647243614773_i8,  236376706414375340_i8, 1015589373787052514_i8, &
  268621657547832135_i8,  936656358317995402_i8,  116747588348763845_i8,   64770150301416667_i8, &
  514495289052768103_i8,  806710593290884467_i8,  169298124698392326_i8,  895946956174000112_i8, &
 1078757683942961052_i8,  730408099691437407_i8,   13776739489363561_i8,  163833831116188930_i8, &
  445737724895722838_i8, 1002406075704716157_i8,  695943385525346688_i8,  729002796758321584_i8, &
  130031463651033240_i8,  559849988058578030_i8,  583194088554564740_i8, 1007296762864035961_i8, &
  176295887636174810_i8,  199917550306618071_i8,  306795835725430548_i8,  970917250543570455_i8, &
  557119029881659578_i8,   77160043500812972_i8,  644143052287219382_i8,  469666497708195358_i8, &
  317888593403527561_i8,  557093423854158498_i8, 1062833461582812162_i8,  409985498047286872_i8, &
  312579213883563332_i8,  934033081311938469_i8,  720474032265994168_i8,  102530460525320256_i8, &
  230586131385840215_i8, 1112806546357160259_i8,  744121117221372478_i8,  138108319215577446_i8, &
  737556465754963983_i8,   95640134533370931_i8,  272039012822844559_i8,  754433461215736947_i8, &
 1093407507414764170_i8, 1070235303229018952_i8, 1015324187236450210_i8,  679433684068918737_i8, &
  243280381607049812_i8,  443424102427212128_i8,  343909206708904527_i8,  553472606371710800_i8, &
  663272017680010338_i8,  690248097742978496_i8,  448160971146339261_i8,  923638959745392204_i8, &
  489214530723737168_i8,  178352515827753204_i8,  562157254933348505_i8,   16685558365795131_i8, &
   25867117595293706_i8,  901963468246502733_i8,  724327605397766855_i8,  109784923027360115_i8, &
  148504676508363654_i8,   65263612537350149_i8,  352781563386556046_i8, 1082910030622490114_i8/
data (mpegammacon(i), i =  1000,  1116) / &
  162605569964002407_i8,   40130460138681637_i8,   55640168176827995_i8,  345569825266110374_i8, &
  281863194303468911_i8,  267998529385266510_i8,  587017482690214802_i8,  889230313991524302_i8, &
  575010872716361961_i8,  417981775881861267_i8, 1023133833652038989_i8,  398035791072563612_i8, &
  401555093146076366_i8,  241479748549441083_i8, 1057178362697189258_i8,  176350238359490528_i8, &
  939955621972608358_i8,  823494246298524110_i8,  256146417344501293_i8,  581774358592793427_i8, &
 1080948294860603777_i8,  645393943037229855_i8,  680452448894659711_i8,   81807481255588932_i8, &
  984131915986141853_i8,  633041904107993408_i8,  654890098474597196_i8,  797357746776100740_i8, &
 1027067607276637660_i8,  341272813153538607_i8,  447398987798064147_i8,  245759169880115702_i8, &
  434759933558056218_i8,  525270843839534452_i8, 1121073072978377940_i8,  222782165970477588_i8, &
  189139101839306435_i8,  508532643527340238_i8,  340979965849919435_i8,  841229410991306759_i8, &
  834454878496374820_i8,  324285793167805600_i8,  121331924095458438_i8,  427400425672600450_i8, &
 1014219721090848685_i8,  730208548507924916_i8,  367528971325272375_i8,  777479132373837111_i8, &
  242315505614328575_i8,  108873393500182068_i8,   58882954202155872_i8,  347209087244555055_i8, &
 1048902453124477487_i8,  671789232077624592_i8,  964169691513953358_i8,  849662557191963898_i8, &
  427440085429851197_i8,  383805069679390223_i8,  781299902151674293_i8,  355663361438381343_i8, &
  989673266275171705_i8,  313663539302272968_i8,  141691654266832215_i8,    3954965905717179_i8, &
  787493985509514659_i8,  475918748445943369_i8,  904292132949020222_i8, 1098531370892896368_i8, &
  866548714840721744_i8,  869415584151913374_i8,  621542941688532327_i8,  653842301536525834_i8, &
 1073330246200921965_i8,  996780712160258487_i8, 1107515170003587484_i8,  236477584935961624_i8, &
  112457406160645685_i8,  795908699388922990_i8, 1036113464272977907_i8,   47717454592653227_i8, &
  426054062595998972_i8, 1118722977254512321_i8,  148432094566334945_i8,  902140132114953081_i8, &
  390242585981130781_i8,  743234857956641642_i8,  932873365129193551_i8,  681909952461289247_i8, &
  175766665580630031_i8,  163297199488290060_i8,  687748240527905520_i8,  227463734076241797_i8, &
  603812331665215935_i8,  986578420901939853_i8,  498195273398166069_i8,  806100481918870901_i8, &
  351685465571530346_i8,  668842737720686894_i8,     835172320078235_i8, 1084877595156355282_i8, &
  437019800648671186_i8,  359478995733847269_i8,  867705251723435639_i8,  786057563927845685_i8, &
  932147109105251660_i8,  162386106703088875_i8, 1093462993704630012_i8,  700883703477362638_i8, &
  297472621637314614_i8,  270483390017205115_i8,  682914563009860091_i8,  233655999241551717_i8, &
  565322608565965141_i8, 1040979220969367247_i8,  694895770700096890_i8,                   0_i8, &
                   0_i8/

end module mpfuna


